/*
 * File:          sidlx_rmi_SimpleServer_Impl.c
 * Symbol:        sidlx.rmi.SimpleServer-v0.1
 * Symbol Type:   class
 * Babel Version: 0.99.0
 * Description:   Server-side implementation for sidlx.rmi.SimpleServer
 * 
 * WARNING: Automatically generated; only changes within splicers preserved
 * 
 */

/*
 * DEVELOPERS ARE EXPECTED TO PROVIDE IMPLEMENTATIONS
 * FOR THE FOLLOWING METHODS BETWEEN SPLICER PAIRS.
 */

/*
 * Symbol "sidlx.rmi.SimpleServer" (version 0.1)
 * 
 * A multi-threaded base class for simple network servers.
 * 
 * This server takes the following flags:
 * 1: verbose output (to stdout)
 */

#include "sidlx_rmi_SimpleServer_Impl.h"

#line 28 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
/* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer._includes) */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <errno.h>
#include <sys/param.h>
#include "sidlx_common.h"
#include "sidlx_rmi_Common.h"
#include "sidl_rmi_NetworkException.h"
#include "sidl_String.h"
#include <pthread.h>

#define LISTENQ 1024
#define MAXLINE 1023

static int nPool = 0;
static pthread_mutex_t poolLock = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t  poolCond = PTHREAD_COND_INITIALIZER;
static int nBusy = 0;
static int haveWork = 0;
static int servingConnections = 1;
static sidlx_rmi_SimpleServer arg0 = NULL;
static sidlx_rmi_Socket arg1 = NULL;


/**
 * This function is executed by child threads dealing with server connections.
 * It just runs the simpleServer serviceRequest, and cleans up after itself on exit.
 */
static void * threadFunc(void *arg) {
  sidlx_rmi_SimpleServer self = NULL;
  sidlx_rmi_Socket ac_sock = NULL;

  sidl_BaseInterface _ex = NULL;
  sidl_BaseInterface _ex2 = NULL;
  sidl_SIDLException e_x_p = NULL;
  /* stacktrace(25); */

  pthread_detach(pthread_self());

  while(1) {
    /* Try to grab some work */
    pthread_mutex_lock(&poolLock);
    while(! haveWork) {
      pthread_cond_wait(&poolCond,&poolLock);
    }
    nBusy++;                    /* Mark a busy thread */
    self = arg0;
    ac_sock = arg1;
    haveWork = 0;               /* Mark work taken */
    pthread_cond_broadcast(&poolCond);
    pthread_mutex_unlock(&poolLock);

    sidlx_rmi_SimpleServer_serviceRequest( self, ac_sock, &_ex );
    SIDL_CHECK(_ex);

    sidlx_rmi_Socket_deleteRef(ac_sock,&_ex); SIDL_CHECK(_ex);

    /* Mark as not busy anymore */
    pthread_mutex_lock(&poolLock);
    nBusy--;
    pthread_mutex_unlock(&poolLock);
  }
  return NULL;
 EXIT:
  /* Assume that only coming here when busy and lock is unlocked */
  pthread_mutex_lock(&poolLock);
  nBusy--;
  nPool--;
  pthread_mutex_unlock(&poolLock);

  sidlx_rmi_Socket_deleteRef(ac_sock,&_ex2); SIDL_CHECK(_ex2);
  e_x_p = sidl_SIDLException__cast(_ex,&_ex2); SIDL_CHECK(_ex2);
  printf("Error in thread! %s\n", sidl_SIDLException_getNote(e_x_p,&_ex2));
  SIDL_CLEAR(_ex);
  return NULL;

  /*  sidl_BaseInterface ex = NULL;
      sidl_BaseException s_b_e = NULL;
      char *str = NULL;
      void ** args = (void**) arg; 
      pthread_detach(pthread_self());
      sidlx_rmi_SimpleServer_serviceRequest( (sidlx_rmi_SimpleServer)args[0], 
      (sidlx_rmi_Socket) args[1],
      &ex ); SIDL_CHECK(ex);
      
      sidlx_rmi_Socket_deleteRef((sidlx_rmi_Socket) args[1], &ex); SIDL_CHECK(ex);
      free(arg);
      close((int)arg);
      return NULL;
      EXIT:
      {
      sidl_BaseInterface throwaway = NULL;
      s_b_e = sidl_BaseException__cast(ex, &throwaway);
      str = sidl_BaseException_getNote(s_b_e, &throwaway);
      printf("ERROR! %s \n",str);
      return NULL;
      }
  */
}

/**
 * This function is executed by the thread that acts as server, accepting
 * connections and spawning children to deal with them.  It does not 
 * detatch itself because the main thread may need to join to it to prevent
 * premature exit.
 * It is also designed to work in a nonthreaded enviornment, but I haven't
 * tested that.
 */


static void * serverFunc(void *arg) {

  sidlx_rmi_SimpleServer self = (sidlx_rmi_SimpleServer)arg;
  sidlx_rmi_Socket ac_sock = NULL;
  sidl_BaseInterface _ex = NULL;
  sidl_BaseInterface ex2 = NULL;
  sidl_SIDLException e_x_p = NULL;
  struct sidlx_rmi_SimpleServer__data *dptr=sidlx_rmi_SimpleServer__get_data(self);

  /* stacktrace(25); */

  if(dptr) {    
    while(servingConnections) {
      pthread_t tid = 0;
      ac_sock = sidlx_rmi_ServerSocket_accept(dptr->s_sock, &_ex); SIDL_CHECK(_ex);
      /* See if we have any bored workers */
      if((dptr->d_flags) & 1) { /* If the "verbose" flag is set*/
	printf("SimpleServer: connection\n");
      }
      
      pthread_mutex_lock(&poolLock);
      if (nBusy == nPool) {
        pthread_create(&tid,0,threadFunc,(void*)0);
        nPool++;
      }

      /* Make sure we don't have pending work */
      while(haveWork) {
        pthread_cond_wait(&poolCond,&poolLock);
      }
      haveWork = 1;
      arg0 = self;
      arg1 = ac_sock;
      pthread_cond_broadcast(&poolCond);
      pthread_mutex_unlock(&poolLock);

      }
      return NULL;
  }
  SIDL_THROW(_ex, sidl_rmi_NetworkException, "Simple Server not initialized");
 EXIT:
  e_x_p = sidl_SIDLException__cast(_ex, &ex2);
  printf("Error in thread! %s\n", sidl_SIDLException_getNote(e_x_p,ex2));
  SIDL_CLEAR(_ex);
  return NULL;  

  /*  sidlx_rmi_SimpleServer self = (sidlx_rmi_SimpleServer)arg;
      sidlx_rmi_Socket ac_sock = NULL;
      sidl_BaseInterface ex = NULL;
      sidl_BaseException s_b_e = NULL;
      char *str = NULL;
      struct sidlx_rmi_SimpleServer__data *dptr=sidlx_rmi_SimpleServer__get_data(self);
      if(dptr) {    
      for(;;) {
       pthread_t tid = 0;
       void** args = NULL;
       ac_sock = sidlx_rmi_ServerSocket_accept(dptr->s_sock, &ex); SIDL_CHECK(ex);
      
       args = malloc(sizeof(void*)*3);
       args[0] = (void*)self;
       args[1] = (void*)ac_sock;
       pthread_create(&tid, NULL,(void * (*)(void *)) &threadFunc, (void*)args);
       
       if((dptr->d_flags) & 1) { 
       printf("SimpleServer: connection\n");
       }
       sidlx_rmi_SimpleServer_serviceRequest( self, ac_sock, &ex );SIDL_CHECK(ex);
       
       }
       return NULL;
       }
       
       printf("ERROR! Simple Server not initialized");
       return NULL;
       EXIT:
       {
       sidl_BaseInterface throwaway = NULL;
       s_b_e = sidl_BaseException__cast(ex, &throwaway);
       str = sidl_BaseException_getNote(s_b_e, &throwaway);
       printf("ERROR! %s \n",str);
       return NULL;
       }
  */ 

}


/* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer._includes) */
#line 229 "sidlx_rmi_SimpleServer_Impl.c"

/*
 * Static class initializer called exactly once before any user-defined method is dispatched
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer__load"

#ifdef __cplusplus
extern "C"
#endif
void
impl_sidlx_rmi_SimpleServer__load(
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 244 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer._load) */
  /* Insert the implementation of the static class initializer method here... */
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer._load) */
#line 250 "sidlx_rmi_SimpleServer_Impl.c"
}
/*
 * Class constructor called when the class is created.
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer__ctor"

#ifdef __cplusplus
extern "C"
#endif
void
impl_sidlx_rmi_SimpleServer__ctor(
  /* in */ sidlx_rmi_SimpleServer self,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 264 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer._ctor) */
  struct sidlx_rmi_SimpleServer__data *dptr;
  dptr = malloc(sizeof(struct sidlx_rmi_SimpleServer__data));
  dptr->s_sock = sidlx_rmi_ServerSocket__create(_ex); SIDL_CHECK(*_ex);
  sidlx_rmi_SimpleServer__set_data(self, dptr);
 EXIT:
  return;
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer._ctor) */
#line 277 "sidlx_rmi_SimpleServer_Impl.c"
}

/*
 * Special Class constructor called when the user wants to wrap his own private data.
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer__ctor2"

#ifdef __cplusplus
extern "C"
#endif
void
impl_sidlx_rmi_SimpleServer__ctor2(
  /* in */ sidlx_rmi_SimpleServer self,
  /* in */ void* private_data,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 291 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer._ctor2) */
  /* Insert-Code-Here {sidlx.rmi.SimpleServer._ctor2} (special constructor method) */
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer._ctor2) */
#line 301 "sidlx_rmi_SimpleServer_Impl.c"
}
/*
 * Class destructor called when the class is deleted.
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer__dtor"

#ifdef __cplusplus
extern "C"
#endif
void
impl_sidlx_rmi_SimpleServer__dtor(
  /* in */ sidlx_rmi_SimpleServer self,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 311 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer._dtor) */
  struct sidlx_rmi_SimpleServer__data * data = sidlx_rmi_SimpleServer__get_data( self );
  if (data) {
    sidlx_rmi_ServerSocket_deleteRef(data->s_sock, _ex);
    free((void*) data);
  }
  sidlx_rmi_SimpleServer__set_data( self, NULL );
  return;
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer._dtor) */
#line 329 "sidlx_rmi_SimpleServer_Impl.c"
}

/*
 * set which port number to bind to.  This request is delivered in the form of a URL.
 * For example:
 * simhandle://localhost:9999/ will bind to port 9999.  To "Simple Protocol" only
 * the port number is actually important.
 * It is also possible to request a port in a range.  For example:
 * simhandle://localhost:9000-9999/ will bind to a port between 9000 and 9999.
 * The return value is the port actually bound to.  An excpetion is returned on failure, 
 * and the port number returned, if any, is invalid.  
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer_init"

#ifdef __cplusplus
extern "C"
#endif
int32_t
impl_sidlx_rmi_SimpleServer_init(
  /* in */ sidlx_rmi_SimpleServer self,
  /* in */ const char* url,
  /* in */ int32_t flags,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 347 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer.init) */
  struct sidlx_rmi_SimpleServer__data *dptr=sidlx_rmi_SimpleServer__get_data(self);
  char* myurl = NULL;
  char* protocol = NULL;
  char* server = NULL;
  int start_port = 0;
  int end_port = 0; 
  int port = 0; 
  char* objectID= NULL;
  
  if(dptr && url) {
    dptr->d_flags = flags;
    myurl = sidl_String_strdup( url );
    sidlx_parseURL(myurl, &protocol, &server, &start_port, &end_port, &objectID, _ex);SIDL_CHECK(*_ex);

    if(end_port == 0) {
      end_port = start_port;
    } else if(start_port > end_port) {   /* If the range is backwards, flip it*/
      int tmp_port = end_port;
      end_port = start_port;
      start_port = tmp_port;
    }

    /* Check all the ports in the inclusive range.  If the port was not useable 
     * (an exception was thrown), and it wasn't the last port in the range,
     * try again.  If it WAS the last port in the range, let the exception 
     * propogate up. */ 
    for(port = start_port; port <= end_port; ++port) {
      sidlx_rmi_ServerSocket_init(dptr->s_sock, port, _ex); 
      if(*_ex && port < end_port) {  
	SIDL_CLEAR(*_ex);
	continue;
      }
      dptr->d_port = port;
      break;
    }
  }

 EXIT:
  if(myurl) {free((void*)myurl);}
  if(protocol) {free((void*)protocol);}
  if(server) {free((void*)server);}
  if(objectID) {free((void*)objectID);}
  if(*_ex) {  /* Little hack to return 0 if an exception was thrown*/
    return 0;
  }
  return port;
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer.init) */
#line 406 "sidlx_rmi_SimpleServer_Impl.c"
}

/*
 * get the port that this Server is bound to
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer_getPort"

#ifdef __cplusplus
extern "C"
#endif
int32_t
impl_sidlx_rmi_SimpleServer_getPort(
  /* in */ sidlx_rmi_SimpleServer self,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 413 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer.getPort) */
  struct sidlx_rmi_SimpleServer__data *dptr=sidlx_rmi_SimpleServer__get_data(self);
  
  if(dptr) {
    return dptr->d_port;
  }
  return 0;
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer.getPort) */
#line 434 "sidlx_rmi_SimpleServer_Impl.c"
}

/*
 * get the flags for this server is configured with
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer_getFlags"

#ifdef __cplusplus
extern "C"
#endif
int32_t
impl_sidlx_rmi_SimpleServer_getFlags(
  /* in */ sidlx_rmi_SimpleServer self,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 439 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer.getFlags) */
  /* Insert-Code-Here {sidlx.rmi.SimpleServer.getFlags} (getFlags method) */
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer.getFlags) */
#line 457 "sidlx_rmi_SimpleServer_Impl.c"
}

/*
 * get the network name of this computer
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer_getServerName"

#ifdef __cplusplus
extern "C"
#endif
char*
impl_sidlx_rmi_SimpleServer_getServerName(
  /* in */ sidlx_rmi_SimpleServer self,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 460 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer.getServerName) */
  char name[MAXHOSTNAMELEN]; /*= sidl_String_alloc(MAXHOSTNAMELEN);*/
  if(gethostname(name, MAXHOSTNAMELEN) == 0) { /*Returns 0 on success*/
    return sidlx_rmi_Common_getCanonicalName(name, _ex);
  }
  /*sidl_String_free(name);*/
  return NULL;
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer.getServerName) */
#line 485 "sidlx_rmi_SimpleServer_Impl.c"
}

/*
 * get the full URL for exporting objects
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer_getServerURL"

#ifdef __cplusplus
extern "C"
#endif
char*
impl_sidlx_rmi_SimpleServer_getServerURL(
  /* in */ sidlx_rmi_SimpleServer self,
  /* in */ const char* objID,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 487 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer.getServerURL) */
  /* TODO: Is there anything we can do here?*/
  return NULL;
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer.getServerURL) */
#line 510 "sidlx_rmi_SimpleServer_Impl.c"
}

/*
 * run the server (must have port specified first), if a threaded server,
 * returns the tid of the server thread for joining.
 */

#undef __FUNC__
#define __FUNC__ "impl_sidlx_rmi_SimpleServer_run"

#ifdef __cplusplus
extern "C"
#endif
int64_t
impl_sidlx_rmi_SimpleServer_run(
  /* in */ sidlx_rmi_SimpleServer self,
  /* out */ sidl_BaseInterface *_ex)
{
  *_ex = 0;
#line 510 "../../../babel_0_99_0_branch/runtime/sidlx/sidlx_rmi_SimpleServer_Impl.c"
  /* DO-NOT-DELETE splicer.begin(sidlx.rmi.SimpleServer.run) */
  pthread_t tid = 0;
  sidlx_rmi_SimpleServer_addRef(self, _ex); SIDL_CHECK(*_ex);
  pthread_create(&tid, NULL, (void * (*)(void *))&serverFunc, (void*)self);
  return (int64_t)tid;
 EXIT:
  return 0;
  /* DO-NOT-DELETE splicer.end(sidlx.rmi.SimpleServer.run) */
#line 539 "sidlx_rmi_SimpleServer_Impl.c"
}
/* Babel internal methods, Users should not edit below this line. */
struct sidl_BaseClass__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidl_BaseClass(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidl_BaseClass__connectI(url, ar, _ex);
}
struct sidl_BaseClass__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidl_BaseClass(void* bi,
  sidl_BaseInterface* _ex) {
  return sidl_BaseClass__cast(bi, _ex);
}
struct sidl_BaseInterface__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidl_BaseInterface(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidl_BaseInterface__connectI(url, ar, _ex);
}
struct sidl_BaseInterface__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidl_BaseInterface(void* bi,
  sidl_BaseInterface* _ex) {
  return sidl_BaseInterface__cast(bi, _ex);
}
struct sidl_ClassInfo__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidl_ClassInfo(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidl_ClassInfo__connectI(url, ar, _ex);
}
struct sidl_ClassInfo__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidl_ClassInfo(void* bi,
  sidl_BaseInterface* _ex) {
  return sidl_ClassInfo__cast(bi, _ex);
}
struct sidl_RuntimeException__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidl_RuntimeException(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidl_RuntimeException__connectI(url, ar, _ex);
}
struct sidl_RuntimeException__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidl_RuntimeException(void* bi,
  sidl_BaseInterface* _ex) {
  return sidl_RuntimeException__cast(bi, _ex);
}
struct sidl_io_Serializable__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidl_io_Serializable(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidl_io_Serializable__connectI(url, ar, _ex);
}
struct sidl_io_Serializable__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidl_io_Serializable(void* bi,
  sidl_BaseInterface* _ex) {
  return sidl_io_Serializable__cast(bi, _ex);
}
struct sidl_rmi_BaseServer__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidl_rmi_BaseServer(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidl_rmi_BaseServer__connectI(url, ar, _ex);
}
struct sidl_rmi_BaseServer__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidl_rmi_BaseServer(void* bi,
  sidl_BaseInterface* _ex) {
  return sidl_rmi_BaseServer__cast(bi, _ex);
}
struct sidl_rmi_ServerInfo__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidl_rmi_ServerInfo(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidl_rmi_ServerInfo__connectI(url, ar, _ex);
}
struct sidl_rmi_ServerInfo__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidl_rmi_ServerInfo(void* bi,
  sidl_BaseInterface* _ex) {
  return sidl_rmi_ServerInfo__cast(bi, _ex);
}
struct sidlx_rmi_SimpleServer__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidlx_rmi_SimpleServer(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidlx_rmi_SimpleServer__connectI(url, ar, _ex);
}
struct sidlx_rmi_SimpleServer__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidlx_rmi_SimpleServer(void* bi,
  sidl_BaseInterface* _ex) {
  return sidlx_rmi_SimpleServer__cast(bi, _ex);
}
struct sidlx_rmi_Socket__object* 
  impl_sidlx_rmi_SimpleServer_fconnect_sidlx_rmi_Socket(const char* url,
  sidl_bool ar, sidl_BaseInterface *_ex) {
  return sidlx_rmi_Socket__connectI(url, ar, _ex);
}
struct sidlx_rmi_Socket__object* 
  impl_sidlx_rmi_SimpleServer_fcast_sidlx_rmi_Socket(void* bi,
  sidl_BaseInterface* _ex) {
  return sidlx_rmi_Socket__cast(bi, _ex);
}
